<?php

namespace DigiComp\PhpCodesniffer\DigiComp\Sniffs\Commenting;

use PHP_CodeSniffer\Files\File;

class VariableCommentSniff extends \PHP_CodeSniffer\Standards\Squiz\Sniffs\Commenting\VariableCommentSniff
{
    public function processMemberVar(File $phpcsFile, $stackPtr)
    {
        $tokens = $phpcsFile->getTokens();
        $ignore = [
            \T_PUBLIC,
            \T_PRIVATE,
            \T_PROTECTED,
            \T_VAR,
            \T_STATIC,
            \T_WHITESPACE,
            \T_STRING,
            \T_NS_SEPARATOR,
            \T_NULLABLE,
        ];

        $commentEnd = $phpcsFile->findPrevious($ignore, ($stackPtr - 1), null, true);
        // this error is copied from above, as it comes with a needed early return
        if (
            $commentEnd === false
            || ($tokens[$commentEnd]['code'] !== \T_DOC_COMMENT_CLOSE_TAG
                && $tokens[$commentEnd]['code'] !== \T_COMMENT)
        ) {
            $phpcsFile->addError('Missing member variable doc comment', $stackPtr, 'Missing');
            return;
        }
        $commentStart = $tokens[$commentEnd]['comment_opener'];

        $foundVar = null;
        foreach ($tokens[$commentStart]['comment_tags'] as $tag) {
            if (
                $tokens[$tag]['content'] === '@var'
                || $tokens[$tag]['content'] === '@inheritDoc'
            ) {
                $foundVar = $tag;
            }
        }

        // The @var tag is the only one we require.
        if ($foundVar === null) {
            $error = 'Missing @var or @inheritDoc tag in member variable comment';
            $phpcsFile->addWarning($error, $commentEnd, 'MissingVarOrInheritDoc');
            return;
        }

        parent::processMemberVar($phpcsFile, $stackPtr);
    }
}
